﻿using System;
using System.Collections.Generic;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.RealVectorEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.MultiObjectiveTestFunctions {
  [Item("Fonseca", "Fonseca and Flemming function from // https://en.wikipedia.org/wiki/Test_functions_for_optimization [30.11.2015]")]
  [StorableClass]
  public class Fonseca : MultiObjectiveTestFunction {

    public override double[,] Bounds(int objectives) {
        return new double[,] { { -4, 4 } };
    }

    public override bool[] Maximization(int objecitves) {
      return new bool[2];
    }

    public override int MaximumSolutionLength {
      get {return int.MaxValue; }
    }

    public override int MinimumSolutionLength {
      get { return 1; }
    }

    public override int MinimumObjectives {
      get { return 2;  }
    }

    public override int MaximumObjectives {
      get { return 2; }
    }


    public override IEnumerable<double[]> OptimalParetoFront(int objectives) {
        return PFStore.get(this.ItemName);
    }
    public override double BestKnownHypervolume(int objectives) {
        return new Hypervolume(ReferencePoint(objectives), Maximization(2)).GetHypervolume(OptimalParetoFront(objectives));
    }

    public override double[] ReferencePoint(int objectives) {
      return new double[] { 11, 11 };
    }

    [StorableConstructor]
    protected Fonseca(bool deserializing) : base(deserializing) { }
    protected Fonseca(Fonseca original, Cloner cloner) : base(original, cloner) { }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new Fonseca(this, cloner);
    }
    public Fonseca() : base() { }


    public override double[] Evaluate(RealVector r, int objectives) {
      if (objectives != 2) throw new ArgumentException("The Fonseca problem must always have 2 objectives");
      double f0 = 0.0, aux = 1.0 / Math.Sqrt(r.Length);

      //objective1
      for (int i = 0; i < r.Length; i++) {
        double d = r[i] - aux;
        f0 += d * d;
      }
      f0 = 1 - Math.Exp(-f0);

      //objective2
      double f1 = 0.0;
      for (int i = 0; i < r.Length; i++) {
        double d = r[i] + aux;
        f1 += d * d;
      }
      f1 = 1 - Math.Exp(-f1);

      double[] res = { f0, f1 };
      return res;
    }

  }
}
